@@ -48,8 +48,15 @@ static void close_comms(void)
 
 static int init_comms(void)
 {
-	struct sockaddr_un addr;
 	mode_t old_mask;
+	struct sockaddr_un addr = { .sun_family = AF_UNIX };
+
+	if (!dm_strncpy(addr.sun_path, SINGLENODE_CLVMD_SOCKNAME,
+			sizeof(addr.sun_path))) {
+		DEBUGLOG("%s: singlenode socket name too long.",
+			 SINGLENODE_CLVMD_SOCKNAME);
+		return -1;
+	}
 
 	close_comms();
 
@@ -67,11 +74,6 @@ static int init_comms(void)
 		goto error;
 	}
 
-	memset(&addr, 0, sizeof(addr));
-	memcpy(addr.sun_path, SINGLENODE_CLVMD_SOCKNAME,
-	       sizeof(SINGLENODE_CLVMD_SOCKNAME));
-	addr.sun_family = AF_UNIX;
-
 	if (bind(listen_fd, (struct sockaddr *)&addr, sizeof(addr)) < 0) {
 		DEBUGLOG("Can't bind local socket: %s\n", strerror(errno));
 		goto error;
@@ -134,7 +136,7 @@ static int _csid_from_name(char *csid, const char *name)
 
 static int _name_from_csid(const char *csid, char *name)
 {
-	sprintf(name, "SINGLENODE");
+	strcpy(name, "SINGLENODE");
 	return 0;
 }
 
@@ -216,8 +218,8 @@ maybe_retry:
 		goto retry;
 	}
 bad:
-	DEBUGLOG("Failed to lock resource %s\n", resource);
 	pthread_mutex_unlock(&_lock_mutex);
+	DEBUGLOG("Failed to lock resource %s\n", resource);
 
 	return 1; /* fail */
 }
@@ -249,9 +251,9 @@ static int _unlock_resource(const char *resource, int lockid)
 	}
 
 	dm_hash_remove(_locks, resource);
-	dm_free(lck);
 	pthread_cond_broadcast(&_lock_cond); /* wakeup waiters */
 	pthread_mutex_unlock(&_lock_mutex);
+	dm_free(lck);
 
 	return 0;
 }
@@ -282,9 +284,7 @@ static int _cluster_send_message(const void *buf, int msglen,
 
 static int _get_cluster_name(char *buf, int buflen)
 {
-	strncpy(buf, "localcluster", buflen);
-	buf[buflen - 1] = 0;
-	return 0;
+	return dm_strncpy(buf, "localcluster", buflen) ? 0 : 1;
 }
 
 static struct cluster_ops _cluster_singlenode_ops = {
@@ -311,6 +311,6 @@ struct cluster_ops *init_singlenode_cluster(void)
 {
 	if (!_init_cluster())
 		return &_cluster_singlenode_ops;
-	else
-		return NULL;
+
+	return NULL;
 }
