@@ -16,7 +16,7 @@ which mkfs.ext2 || skip
 aux prepare_vg 5
 aux prepare_dmeventd
 
-lvcreate -m 3 --ig -L 1 -n 4way $vg
+lvcreate -aey --type mirror -m 3 --ignoremonitoring -L 1 -n 4way $vg
 lvchange --monitor y $vg/4way
 aux disable_dev "$dev2" "$dev4"
 mkfs.ext2 $DM_DEV_DIR/$vg/4way
