@@ -24,9 +24,7 @@ pvcreate "$dev1"
 pvcreate --metadatacopies 0 "$dev2"
 
 # "check lv snapshot"
-vgcreate -c n $vg "$dev1" "$dev2"
-lvcreate -n $lv -l 60%FREE $vg
+vgcreate $vg "$dev1" "$dev2"
+lvcreate -aey -n $lv -l 60%FREE $vg
 lvcreate -s -n $lv2 -l 10%FREE $vg/$lv
-pvdisplay
-lvdisplay
 vgremove -f $vg
