@@ -179,6 +179,7 @@ static struct logical_volume *_set_up_pvmove_lv(struct cmd_context *cmd,
 						unsigned *exclusive)
 {
 	struct logical_volume *lv_mirr, *lv;
+	struct lv_segment *seg;
 	struct lv_list *lvl;
 	uint32_t log_count = 0;
 	int lv_found = 0;
@@ -215,35 +216,56 @@ static struct logical_volume *_set_up_pvmove_lv(struct cmd_context *cmd,
 		}
 		if (lv_is_origin(lv) || lv_is_cow(lv)) {
 			lv_skipped = 1;
-			log_print("Skipping snapshot-related LV %s", lv->name);
+			log_print_unless_silent("Skipping snapshot-related LV %s", lv->name);
+			continue;
+		}
+		if (lv_is_raid_type(lv)) {
+			seg = first_seg(lv);
+			if (seg_is_raid(seg)) {
+				lv_skipped = 1;
+				log_print_unless_silent("Skipping %s LV %s",
+							seg->segtype->ops->name(seg),
+							lv->name);
+				continue;
+			}
+			lv_skipped = 1;
+			log_print_unless_silent("Skipping RAID sub-LV %s",
+						lv->name);
 			continue;
 		}
 		if (lv->status & MIRRORED) {
 			lv_skipped = 1;
-			log_print("Skipping mirror LV %s", lv->name);
+			log_print_unless_silent("Skipping mirror LV %s", lv->name);
 			continue;
 		}
 		if (lv->status & MIRROR_LOG) {
 			lv_skipped = 1;
-			log_print("Skipping mirror log LV %s", lv->name);
+			log_print_unless_silent("Skipping mirror log LV %s", lv->name);
 			continue;
 		}
 		if (lv->status & MIRROR_IMAGE) {
 			lv_skipped = 1;
-			log_print("Skipping mirror image LV %s", lv->name);
+			log_print_unless_silent("Skipping mirror image LV %s", lv->name);
+			continue;
+		}
+		if (lv_is_thin_volume(lv) || lv_is_thin_pool(lv)) {
+			lv_skipped = 1;
+			log_print_unless_silent("Skipping thin%s LV %s",
+						lv_is_thin_pool(lv) ? "-pool" : "",
+						lv->name);
 			continue;
 		}
 		if (lv->status & LOCKED) {
 			lv_skipped = 1;
-			log_print("Skipping locked LV %s", lv->name);
+			log_print_unless_silent("Skipping locked LV %s", lv->name);
 			continue;
 		}
 
 		if (vg_is_clustered(vg) &&
 		    lv_is_active_exclusive_remotely(lv)) {
 			lv_skipped = 1;
-			log_print("Skipping LV %s which is activated "
-				  "exclusively on remote node.", lv->name);
+			log_print_unless_silent("Skipping LV %s which is activated "
+						"exclusively on remote node.", lv->name);
 			continue;
 		}
 
@@ -442,7 +464,7 @@ static int _update_metadata(struct cmd_context *cmd, struct volume_group *vg,
 			 * FIXME Run --abort internally here.
 			 */
 			log_error("ABORTING: Temporary pvmove mirror activation failed. Run pvmove --abort.");
-			goto_out;
+			goto out;
 		}
 	}
 
@@ -479,7 +501,7 @@ static int _set_up_pvmove(struct cmd_context *cmd, const char *pv_name,
 	argv++;
 
 	/* Find PV (in VG) */
-	if (!(pv = find_pv_by_name(cmd, pv_name))) {
+	if (!(pv = find_pv_by_name(cmd, pv_name, 0))) {
 		stack;
 		return EINVALID_CMD_LINE;
 	}
@@ -505,14 +527,13 @@ static int _set_up_pvmove(struct cmd_context *cmd, const char *pv_name,
 	vg = _get_vg(cmd, pv_vg_name(pv));
 	if (vg_read_error(vg)) {
 		release_vg(vg);
-		stack;
-		return ECMD_FAILED;
+		return_ECMD_FAILED;
 	}
 
 	exclusive = _pvmove_is_exclusive(cmd, vg);
 
 	if ((lv_mirr = find_pvmove_lv(vg, pv_dev(pv), PVMOVE))) {
-		log_print("Detected pvmove in progress for %s", pv_name);
+		log_print_unless_silent("Detected pvmove in progress for %s", pv_name);
 		if (argc || lv_name)
 			log_error("Ignoring remaining command line arguments");
 
@@ -661,7 +682,7 @@ static struct volume_group *_get_move_vg(struct cmd_context *cmd,
 	struct volume_group *vg;
 
 	/* Reread all metadata in case it got changed */
-	if (!(pv = find_pv_by_name(cmd, name))) {
+	if (!(pv = find_pv_by_name(cmd, name, 0))) {
 		log_error("ABORTING: Can't reread PV %s", name);
 		/* What more could we do here? */
 		return NULL;
